@@ -32,6 +32,7 @@
 #include "lib/mgmt.h"
 
 #include "src/shared/util.h"
+#include "src/shared/queue.h"
 #include "src/shared/mgmt.h"
 #include "src/shared/gap.h"
 
@@ -39,6 +40,7 @@
 
 struct bt_gap {
 	int ref_count;
+	uint16_t index;
 	struct mgmt *mgmt;
 
 	uint8_t mgmt_version;
@@ -50,8 +52,25 @@ struct bt_gap {
 	bt_gap_ready_func_t ready_handler;
 	bt_gap_destroy_func_t ready_destroy;
 	void *ready_data;
+
+	uint8_t static_addr[6];
+	uint8_t local_irk[16];
+	struct queue *irk_list;
 };
 
+struct irk_entry {
+	uint8_t addr_type;
+	uint8_t addr[6];
+	uint8_t key[16];
+};
+
+static void irk_entry_free(void *data)
+{
+	struct irk_entry *irk = data;
+
+	free(irk);
+}
+
 static void ready_status(struct bt_gap *gap, bool status)
 {
 	gap->mgmt_ready = status;
@@ -130,20 +149,32 @@ static void read_version_complete(uint8_t status, uint16_t length,
 	}
 }
 
-struct bt_gap *bt_gap_new(void)
+struct bt_gap *bt_gap_new_default(void)
+{
+	return bt_gap_new_index(0x0000);
+}
+
+struct bt_gap *bt_gap_new_index(uint16_t index)
 {
 	struct bt_gap *gap;
 
+	if (index == MGMT_INDEX_NONE)
+		return NULL;
+
 	gap = new0(struct bt_gap, 1);
 	if (!gap)
 		return NULL;
 
+	gap->index = index;
+
 	gap->mgmt = mgmt_new_default();
 	if (!gap->mgmt) {
 		free(gap);
 		return NULL;
 	}
 
+	gap->irk_list = queue_new();
+
 	gap->mgmt_ready = false;
 
 	if (!mgmt_send(gap->mgmt, MGMT_OP_READ_VERSION,
@@ -182,6 +213,8 @@ void bt_gap_unref(struct bt_gap *gap)
 	if (gap->ready_destroy)
 		gap->ready_destroy(gap->ready_data);
 
+	queue_destroy(gap->irk_list, irk_entry_free);
+
 	mgmt_unref(gap->mgmt);
 
 	free(gap);
@@ -203,3 +236,45 @@ bool bt_gap_set_ready_handler(struct bt_gap *gap,
 
 	return true;
 }
+
+bool bt_gap_set_static_addr(struct bt_gap *gap, uint8_t addr[6])
+{
+	if (!gap)
+		return false;
+
+	memcpy(gap->static_addr, addr, 6);
+
+	return true;
+}
+
+bool bt_gap_set_local_irk(struct bt_gap *gap, uint8_t key[16])
+{
+	if (!gap)
+		return false;
+
+	memcpy(gap->local_irk, key, 16);
+
+	return true;
+}
+
+bool bt_gap_add_peer_irk(struct bt_gap *gap, uint8_t addr_type,
+					uint8_t addr[6], uint8_t key[16])
+{
+	struct irk_entry *irk;
+
+	if (!gap)
+		return false;
+
+	if (addr_type > BT_GAP_ADDR_TYPE_LE_RANDOM)
+		return false;
+
+	irk = new0(struct irk_entry, 1);
+	if (!irk)
+		return false;
+
+	irk->addr_type = addr_type;
+	memcpy(irk->addr, addr, 6);
+	memcpy(irk->key, key, 16);
+
+	return true;
+}
